#!/usr/bin/env python3

# The code in this file is released under the Creative Commons CC0 license,
# i.e., for unrestricted use and without any warranty,
# see the file license_CC0.txt or https://creativecommons.org/publicdomain/zero/1.0/

from scipy.stats import pearsonr, linregress, spearmanr
import scipy
import matplotlib
import matplotlib.pyplot as plt
import numpy as np

def generateData():
    def addNoiseToData(data, noise_ratio):
        for i, x in enumerate(data):
            noisy_value = x * (1.0 + np.random.uniform(- noise_ratio, + noise_ratio))
            data[i] = noisy_value
        return data

    num_datapoints = 31
    x_points = np.linspace(0, 30, num_datapoints)

    np.random.seed(0)
    series_a = addNoiseToData([15 for _ in x_points], 0.8)
    series_b = addNoiseToData([5 + 20 * np.sin(np.pi/30 * i ) for i in x_points], 0.04)
    series_c = addNoiseToData([30 * i / (1.0 + i) for i in x_points], 0.02)
    series_d = addNoiseToData([15 + 0.25 * i for i in x_points], 0.1)
    series_e = addNoiseToData([i for i in x_points], 0.2)

    return x_points, series_a, series_b, series_c, series_d, series_e


def computePearson(x_points, y_points):
    return pearsonr(x_points, y_points)[0]


def computeSpearmanR(x_points, y_points):
    return spearmanr(x_points, y_points)[0]


def ccc(x_values, y_values):
    # compute the concordance correlation coefficient
    covar = scipy.cov(x_values, y_values)[0][1]  # default is "unbiased N-1"
    xmean = scipy.mean(x_values)
    xvar = scipy.var(x_values, ddof=1)  # need ddof=-1 for "unbiased N-1"
    ymean = scipy.mean(y_values)
    yvar = scipy.var(y_values, ddof=1)
    return (2 * covar) / (xvar + yvar + (xmean - ymean)**2)


def addPlot(ax, x_values, y_values, title):
    vmin = x_values[0]-1
    vmax = x_values[-1]+1
    ax.set_xlim(vmin, vmax)
    ax.set_ylim(vmin, vmax)

    ax.get_xaxis().set_ticklabels([])
    ax.get_yaxis().set_ticklabels([])

    ax.plot(x_values, y_values, 'kx', markersize=3)
    slope, intercept, _, _, _ = linregress(x_values, y_values)
    x_values = [vmin] + list(x_values) + [vmax]
    ax.plot(x_values, [intercept + slope * i for i in x_values], 'k-', lw=1)
    ax.plot(x_values, x_values, 'k:', lw=0.8)
    ax.xaxis.set_ticks(np.arange(0, 40, 10))
    ax.yaxis.set_ticks(np.arange(0, 40, 10))
    ax.set_title(title)
    ax.set_aspect('equal')


def printCCs(x_points, series_a, series_b, series_c, series_d, series_e):
    print(computeSpearmanR(x_points, series_a), computePearson(x_points, series_a), ccc(x_points, series_a))
    print(computeSpearmanR(x_points, series_b), computePearson(x_points, series_b), ccc(x_points, series_b))
    print(computeSpearmanR(x_points, series_c), computePearson(x_points, series_c), ccc(x_points, series_c))
    print(computeSpearmanR(x_points, series_d), computePearson(x_points, series_d), ccc(x_points, series_d))
    print(computeSpearmanR(x_points, series_e), computePearson(x_points, series_e), ccc(x_points, series_e))
    print(computeSpearmanR(x_points, x_points), computePearson(x_points, x_points), ccc(x_points, x_points))  # these should all be one


def plotValues(x_points, series_a, series_b, series_c, series_d, series_e):
    f, axarr = plt.subplots(1, 5, figsize=(7.5,2))
    matplotlib.rcParams.update({'font.size': 9})
    addPlot(axarr[0], x_points, series_a, 'A')
    addPlot(axarr[1], x_points, series_b, 'B')
    addPlot(axarr[2], x_points, series_c, 'C')
    addPlot(axarr[3], x_points, series_d, 'D')
    addPlot(axarr[4], x_points, series_e, 'E')

    plt.tight_layout()
    plt.savefig('correlation.eps')


if __name__ == '__main__':
    x_points, series_a, series_b, series_c, series_d, series_e = generateData()
    printCCs(x_points, series_a, series_b, series_c, series_d, series_e)
    plotValues(x_points, series_a, series_b, series_c, series_d, series_e)
